function Iliad_crystal (sample_run, ei, wbvan_run, mask_run, monovan_run, d_rebin)
% Simplified interface to data reduction on Merlin, MAPS, MARI and LET
%
%   >> iliad (sample_run, ei, wb_van_run)
%
%   >> iliad (sample_run, ei, wb_van_run, mask_run)
%           If mask run number is omitted, or is empty, then the sample_run number is used
%
%   >> iliad (sample_run, ei, wb_van_run, mask_run, mono_van_run)
%
%   >> iliad (sample_run, ei, wb_van_run, mask_run, mono_van_run, d_rebin)
%           Set monovan run number = [] if absolute normalisation not required
%             e.g.   iliad (1434, 50, 1390, [], [], [-20,0.5,45])
%
%
% NOTES:
%   If you want to recompute the white beam run or mask file then you must delete these from the output
%   directories.

% T.G.Perring August 2008, inspired by Rob Bewley's original Iliad
%
% To do:
% (1) 'clean' option - recompute all mask, monovan files
% (2) Write out monochromatic vanadium sum just as as do for mask and white beam files

% =================================================================================================
% User setup:
% =================================================================================================

% Set sample map file(s):
% ------------------------
% (Can comment out entirely - in which case the sample Homering will not be performed. Useful to
% simply create a nask file)

sample_map_file = {...
%     'strip_full_map.map'...
%     'strip_map.map'...
%     'strip_fold_map.map';...
%     'rings_map_all_082.map'...
      'onetoone_doors2-9.map'...
%     'rings_fine_082.map'...
%     'rings_ultrafine_082.map'...
 %   'one2one_082.map'...
    };

% Background range for diag
% --------------------------
% (Can comment out or omit entirely - default is then bmin=12000 ad bmax=18000)

% Goof for Ei=380meV 550Hz:
bmin=12000;
%bmax=18000;
bmax=18000;


% Give sample mass and relative molecular mass
% --------------------------------------------
% Only needed if absolute units required

sample_mass = 32;       % approximate mass
sample_rmm  = 82.03;    % mass per Mn ion

% sample_mass = 32.62;
% sample_rmm  = 50.9415;


% Hard mask file name
% -------------------
% Can be commented out if no hard mask file needed, or equivalently set hard_mask = ''
% if abs(ei-25)<0.5
%     hard_mask = 'hard_mask_ei25.msk';
% else
%     hard_mask = 'hard_mask_gen.msk';
% end


% Set user-defined file locations
% ----------------------------------
% Folder where user wants to read input map, mask files ahead of looking in any instrument default folder(s):
% Can be character string or cell array of character strings; comment out or set ='' to omit

user_input  = 'C:\MAPS_User_Disk\Hayden\LSCO_merlin_sep08';

% Folder where user wants output to be written (.sum, .msk and .spe files):
% Can be character string or cell array of character strings; comment out or set ='' to omit

user_output = 'C:\MAPS_User_Disk\Hayden\LSCO_merlin_sep08';
%user_output = '/data/scratch/sqs43493/MnSi_May2008';

% Folder where raw data is to be found:
data_source = '/data/instrument/MERLIN/RAW';
%data_source = '/sandata/home/wkc26243/Libisis/Libisis_Dev/Tests/matlab_tests/merlin';
%user_output = '/data/scratch/sqs43493/MnSi_May2008';
%user_output = data_source; %'/data/scratch/sqs43493/MnSi_May2008';


% =================================================================================================
% =================================================================================================

% monochromatic vadaium map file and monitor map for sample and mono van runs
monovan_map='inst_maps:::mono_van_map.map';
monitor_map='inst_maps:::merlin_monitors_2-9.map';


% Instrument setup:
%libisis_root = '/usr/local/libisis';     % root directory of libisis installation
libisis_root = which('libisis_init'); % root directory of libisis installation
%'/usr/local/libisis';
inst_abrv_name = 'MER';
inst_full_name = 'merlin';

% Set instrument specific paths and constants
%instrument_files = [libisis_root,'\matlab\Instrument_files\',inst_full_name,'_files\'];
%instrument_files = '/data/instrument/MERLIN/';
MAP_file= which('diag_tubes1to1.map');
% this string identifies path !!!
instrument_files=fileparts(MAP_file);

if exist('user_input','var')&&~isempty(user_input)
    user_input=cellstr(user_input);
    mapdir= {user_input{:}, instrument_files};
    maskdir={user_input{:}, instrument_files};
    nexdir= {user_input{:}, instrument_files};
else
    mapdir= instrument_files;
    maskdir=instrument_files;
    nexdir= instrument_files;
end

if exist('user_output','var')&&~isempty(user_output)
    user_output=cellstr(user_output);
    data_source=cellstr(data_source);
    datadir={user_output{:}, data_source{:}};
else
    data_source=cellstr(data_source);
    datadir=data_source{:};
end

%setup_merlin_inst(datadir,mapdir,nexdir,maskdir);
setup_instrument_data_sources(datadir,mapdir,nexdir,maskdir,'MER');
setup_merlin_homer_defaults;
setup_merlin_diag_defaults;
dso=setup_merlin_data_source;

if exist('sample_map_file','var') && ~isempty(sample_map_file)
    if iscell(sample_map_file)
        sample_map=cell(1,numel(sample_map_file));
        for i=1:numel(sample_map_file)
            sample_map{i}=['inst_maps:::',sample_map_file{i}];
        end
    else
        sample_map{1}=['inst_maps:::',sample_map_file];
    end
    n_map_files = numel(sample_map);
else
    disp('WARNING: No map file given - no Homering will be performed')
    n_map_files = 0;
end



% Get run numbers as a five character string, padded with zeros at front if necessary
sample_run_char= correct_length(sample_run);
wbvan_run_char= correct_length(wbvan_run);

if exist('mask_run','var') && ~isempty(mask_run)
    mask_run_char= correct_length(mask_run);
else
    mask_run = sample_run;
    mask_run_char = sample_run_char;
end

if exist('monovan_run','var') && ~isempty(monovan_run)
    monovan_run_present = true;
    monovan_run_char= correct_length(monovan_run);
else
    monovan_run_present = false;
end

% Check energy transfer bins, if they have been provided.
if exist('d_rebin','var')
    if isnumeric(d_rebin) && isvector(d_rebin) && numel(d_rebin)==3
        if d_rebin(1)>=d_rebin(3) || d_rebin(2)<=0
            error('Check energy binning ')
        end
    else
        error('Energy bins must be a vector [eps_lo, d_eps, eps_hi]')
    end
else
    d_rebin=[-0.15*ei 0.01*ei 0.95*ei];   % output rebinning in energy transfer
end


% =================================================================================================
% Whitebeam vanadium population
% ------------------------------
% Create white beam file only if not already performed

tic
wbvan_file=['inst_data:::' inst_abrv_name wbvan_run_char '.sum'];
if exist(translate_read(wbvan_file),'file')
    disp(['Reading white beam vanadium file ',translate_read(wbvan_file)])
    wb_integrals=read_nxs(wbvan_file,IXTrunfile);
else
    disp(['Creating white beam sum and saving to file ' translate_write(wbvan_file)]);
    wb_integrals = white_beam(dso,wbvan_run);
    write_nxs(wbvan_file,wb_integrals);
end
time=toc;

disp('-----------------------------------------------------------')
disp(['   White beam integrals obtained  (elapsed time: ',num2str(time),' sec)'])
disp('-----------------------------------------------------------')
disp(' ')




% =================================================================================================
% Get mask file
% ------------------------------
% Use pre-existing mask file if requested

tic

if ~exist('bmin','var'), bmin=12000; end
if ~exist('bmax','var'), bmax=18000; end
if bmax<bmin, error('Check background ranges'), end

mask_file=['inst_data:::' inst_abrv_name mask_run_char '.msk'];
if exist(translate_read(mask_file),'file')
    disp(['Reading mask file ',translate_read(mask_file)])
    sam_mask=read_nxs(mask_file,IXTmask);
else
    disp(['Creating mask and saving to file ' translate_write(mask_file)]);
    if exist('hard_mask','var') && ~isempty(hard_mask)
        hard_mask_file=translate_read(['inst_masks:::' hard_mask]);
        if ~exist(hard_mask_file,'file')
            error(['Hard mask file ',hard_mask_file,' does not exist'])
        end
        hard_mask_file=['inst_masks:::' hard_mask];
        sam_mask=diagnose(dso,sample_run_char,wb_integrals,'hardmask',hard_mask_file,...
            'bmin',bmin,'bmax',bmax,'out_nex',mask_file);
    else
        sam_mask=diagnose(dso,sample_run_char,wb_integrals,...
            'bmin',bmin,'bmax',bmax,'out_nex',mask_file);
    end
end
time=toc;

disp('-----------------------------------------------------------')
disp(['   Masked detector list obtained  (elapsed time: ',num2str(time),' sec)'])
disp('-----------------------------------------------------------')
disp(' ')



% =================================================================================================
% Get monochromatic vanadium normalisation
% -----------------------------------------
% There is no need to run diagnose on the monochromatic vanadium run since
% the same wiring has been used for the sample run and the same detector
% should be masked

if monovan_run_present
    tic
    disp ('Get absolute normalisation factor...')
    % Note: monovan_integrals is an IXTrunfile object
    monovan_integrals=mono_van(dso,'sloppy',monovan_run,'ei',ei,'noback','det_map',monovan_map,'det_mask',sam_mask,'mon_map',monitor_map,...
        'solid',wb_integrals,'d_int',[-0.8*ei,0.8*ei],'corr','abs',1,'mass',32.62,'RMM',82);
    abs_norm_factor=monovan_abs(monovan_integrals);
    time=toc;

    disp('-----------------------------------------------------------')
    disp(['   Absolute normalisation factor obtained  (elapsed time: ',num2str(time),' sec)'])
    disp('-----------------------------------------------------------')
    disp(' ')
end


% =================================================================================================
% Homer, optionally with absolute units
% --------------------------------------

for i=1:n_map_files
    tic
    disp(['Performing Homer on sample run for map file ',num2str(i), ' of ', num2str(n_map_files),'...'])
    disp(['  Energy bins: ',num2str(d_rebin(1)),'(',num2str(d_rebin(2)),')',num2str(d_rebin(3))])
    if monovan_run_present
        final_output = mono_sample(dso,'sloppy',sample_run,'ei',ei,'noback','d_rebin',d_rebin,'det_map',sample_map{i},'det_mask',sam_mask,...
            'mon_map',monitor_map,'solid',wb_integrals,'corr','abs',abs_norm_factor,'mass',sample_mass,'RMM',sample_rmm);
    else
        final_output = mono_sample(dso,'sloppy',sample_run,'ei',ei,'noback','d_rebin',d_rebin,'det_map',sample_map{i},'det_mask',sam_mask,...
            'mon_map',monitor_map,'solid',wb_integrals,'corr');
    end
    time=toc;

    disp('-----------------------------------------------------------')
    disp(['   Homer run on sample data  (elapsed time: ',num2str(time),' sec)'])
    if n_map_files>1
        disp(['   for map file ',num2str(i), ' of ', num2str(n_map_files)])
    end
    disp('-----------------------------------------------------------')
    disp(' ')

    % Write out the SPE file
    disp('Write out the SPE file...')
    [dummy,map_name]=fileparts(sample_map_file{i});
    spe_out=['inst_data:::', inst_abrv_name, sample_run_char,'_',map_name, '.spe'];
    runfile2spe(final_output,spe_out);
end




%==================================================================================================
function run_no=correct_length(input)
% Get run number as a five character string, padded with zeros at front if necessary
if input < 0
    error('Run number must be in range 1 - 99999')
elseif input < 10
    run_no=strcat('0000',sprintf('%.0f',round(input)));
elseif input < 100
    run_no=strcat('000',sprintf('%.0f',round(input)));
elseif input < 1000
    run_no=strcat('00',sprintf('%.0f',round(input)));
elseif input < 10000
    run_no=strcat('0',sprintf('%.0f',round(input)));
elseif input < 100000
    run_no=sprintf('%.0f',round(input));
else
    error('Run number must be in range 1 - 99999')
end

