function t_bkgd=chopper_get_bkgd_range(source,moderator,fermi,x2,ei,eps)
% Find a suitable background determination range
%
%   >> t_bkgd=chopper_get_bkgd_range(source,moderator,fermi,x2)         % use ei in fermi object
%   >> t_bkgd=chopper_get_bkgd_range(source,moderator,fermi,x2,ei)      % use eps=0
%   >> t_bkgd=chopper_get_bkgd_range(source,moderator,fermi,x2,ei,eps)
%
% Input:
% -------
%   source      IXTsource object
%   moderator   IXTmoderator object
%   fermi       IXTfermi_chopper object
%   x2          Secondary flightpath/range of secondary flightpaths (m)
%   ei          Incident neutron energy/range of energies (meV). If omitted or empty (i.e. []), then use value in fermi
%   eps         Energy transfer (meV) from chopper spurions to avoid. Default=0 (i.e. avoid elastic line) if omitted
%
% Output:
% -------
%   t_bkgd      [tlo,thi] for background subtraction (microseconds)

% *** should use a better model for resolution from higher order pulses

t_max_frac=0.25;    % Maximum background determination window as fraction of frame
t_min_frac=0.025;   % Minimum background determination window as fraction of frame
t0_frac=0.025;      % Fractional margin by which to set higher time limit of background w.r.t. t=0 pulse
t_frac=0.06;        % Window (full width) to block out as fraction of total time-of-flight
gam_cutoff=4.5;     % notionally 4, but allow for imperfections in the chopper construction

xs=abs(moderator.distance); % moderator-sample distance
x1=abs(fermi.distance);     % chopper-sample distance
x0=xs-x1;                   % moderator-chopper distance
hz=fermi.frequency;         % frequency of source
tmax=10^6/source.frequency;
if ~exist('ei','var') || isempty(ei)
    ei=fermi.energy;
end
if ~exist('eps','var') || isempty(eps)
    eps=0;
end
if any(ei-eps)<=0
    error('Energy transfer must be less than incident energy')
end

% Get ranges of ei, x2
ei_min=min(ei); ei_max=max(ei); ei=[ei_min,ei_max];
x2_min=min(x2); x2_max=max(x2);

% Initial bounding ranges: minimum energy transfer from desired pulse, and the t=0 pulse from next frame
vi = 437.392.*sqrt(ei);
t_elastic=10^6*(x0+x1+x2_max)./vi;
if max(t_elastic)>tmax*(1-t0_frac)
    disp('WARNING: Elastic peak outside time frame')    % from lower bound of ei with maximum x2
    t_bkgd=[];
    return
end

vf = 437.392.*sqrt(ei-eps);
tof= 10^6*((x0+x1)./vi + x2_max./vf);
t_arr=NaN(2,10);    % array to hold ranges to exclude
t_arr(:,1)=[0;max(tof)];
t_arr(:,2)=[tmax*(1-t0_frac);tmax];

% Get array of regions to exclude. Use chopper transmission function to determine if spurions present.
n_pi=0;
n_pk=2;
while true
    n_pi=n_pi+1;
    [t1,ei_prime1] = chopper_spurions (ei_min, hz, n_pi/2, x0, x1, [x2_min,x2_max], 0);
    [t2,ei_prime2] = chopper_spurions (ei_max, hz, n_pi/2, x0, x1, [x2_min,x2_max], 0);
    t=[min([t1,t2]),max([t1,t2])];
    if t(1)<tmax
        phase=logical(mod(n_pi+1,2));
        if gamma(fermi,ei_prime1,phase)<gam_cutoff || gamma(fermi,ei_prime2,phase)<gam_cutoff
            n_pk=n_pk+1;
            if n_pk>size(t_arr,2), t_arr=[t_arr,NaN(size(t_arr))]; end  % double size if not large enough
            t_arr(:,n_pk)=[t(1)*(1-t_frac/2);min(t(2)*(1+t_frac/2),tmax)];
        end
    else
        break
    end
end

% Pack ranges
t_arr=range_union(t_arr(:,1:n_pk));

% Find good intervals
t_int=reshape(t_arr(2:end-1),2,size(t_arr,2)-1);

% Set choice interval
if ~isempty(t_int)
    t_bkgd=t_int(:,end)';
    if t_bkgd(1)<t_bkgd(2)-t_max_frac*tmax;
        t_bkgd(1)=t_bkgd(2)-t_max_frac*tmax;
    elseif t_bkgd(1)>t_bkgd(2)-t_min_frac*tmax;
        t_bkgd=[];
    end
else
    t_bkgd=[];
end

%===============================================================================================
function rout=range_union(r)
% Pack overlapping ranges in [2xn] array
%
%   r       [rlo_1, rlo_2, rlo_3,...; rhi_1, rhi_2, rhi_3,...]
%           It is assumed that rhi_i>=rlo_i, but no assumption is made about the order of 
%           the ranges.
%
%   rout    Output [2xn] array of ranges

nr=size(r,2);
if nr==0
    rout=zeros(2,0);
    return
end
if any(r(2,:)<r(1,:))
    error('Range must have first row large than second row')
end
if nr==1
    rout=r;
    return
end

rout=zeros(size(r));
[dummy,ix]=sort(r(1,:));
r=r(:,ix);  % sorted in order of starting point
rout(:,1)=r(:,1);
i=1;
for j=2:nr
    if r(1,j)>rout(2,i)  % ranges distinct, so accumulate the range
        i=i+1;
        rout(:,i)=r(:,j);
    elseif r(2,j)>rout(2,i) % overlap region, update extent of range if not fully contained
        rout(2,i)=r(2,j);
    end
end

rout=rout(:,1:i);
