function eps=chopper_get_van_integral_range(source,moderator,fermi,x2,ei,t_bkgd)
% Find a suitable background determination range
%
%   >> t_bkgd=chopper_get_bkgd_range(source,moderator,fermi,x2)         % use ei in fermi object
%   >> t_bkgd=chopper_get_bkgd_range(source,moderator,fermi,x2,ei)      % use eps=0
%   >> t_bkgd=chopper_get_bkgd_range(source,moderator,fermi,x2,ei,eps)
%
% Input:
% -------
%   source      IXTsource object
%   moderator   IXTmoderator object
%   fermi       IXTfermi_chopper object
%   x2          Secondary flightpath/range of secondary flightpaths (m)
%   ei          Incident neutron energy/range of incident energies (meV). If omitted or empty (i.e. []), then use value in fermi
%   t_bkgd      Background time range (microseconds). If omitted or empty, then ignored.
%
% Output:
% -------
%   eps         Energy transfer integration range, [eps_lo,eps_hi]
%               The maximum energy transfer to use is determined from location of chopper spurions,
%               and to avoid the background integration range, if given.

% *** should use a better model for resolution from higher order pulses

t0_frac=0.025;      % Fractional margin for maximum energy transfer to avoid t=0 pulse
eps_resln = 0.10;   % fraction of ei_prime for calculation of energy resolution (FWHH) from higher order pulse - make very generous
eps_van = 50;       % maximum energy gain - based on phonon d.o.s. and assuming at ~300K
gam_cutoff=4.5;     % notionally 4, but allow for imperfections in the chopper construction

xs=abs(moderator.distance); % moderator-sample distance
x1=abs(fermi.distance);     % chopper-sample distance
x0=xs-x1;                   % moderator-chopper distance
hz=fermi.frequency;         % frequency of source
tmax=10^6/source.frequency;
if ~exist('ei','var') || isempty(ei)
    ei=fermi.energy;
end
if ~exist('t_bkgd','var') || isempty(t_bkgd)
    t_bkgd=[];
end

% Get ranges of ei, x2
ei_min=min(ei); ei_max=max(ei); ei=[ei_min,ei_max];
x2_min=min(x2); x2_max=max(x2); x2=[x2_min,x2_max];

% Get largest energy transfer. Use chopper transmission function to determine if spurions present.
n_pi=0;
cont=true;
while cont
    n_pi=n_pi+1;
    [dummy,ei_prime] = chopper_spurions ([ei_min,ei_max], hz, n_pi/2, x0, x1, x2_min, 0); % just to get ei_prime for the limits of ei
    eps_prime=sqrt(eps_van^2+(eps_resln*ei_prime).^2);   % use as estimate of FWHH
    [t1,dummy,eps1] = chopper_spurions (ei_min, hz, n_pi/2, x0, x1, [x2_min,x2_max], -eps_prime(1));
    [t2,dummy,eps2] = chopper_spurions (ei_max, hz, n_pi/2, x0, x1, [x2_min,x2_max], -eps_prime(2));
    % there is an inverse relationship between t and eps - the shorter t corresponds to shorter x2 but larger eps
    if t1(1)<tmax*(1-t0_frac) || t2(1)<tmax*(1-t0_frac)
        % for one or both ei, one or both limits of x2 results in eps in the frame
        phase=logical(mod(n_pi+1,2));
%         disp([num2str(min(t1)),'    ',num2str(min(eps1)),'    ',num2str(gamma(fermi,ei_prime(1),phase))])
%         disp([num2str(min(t2)),'    ',num2str(min(eps2)),'    ',num2str(gamma(fermi,ei_prime(2),phase))])
        if any(gamma(fermi,ei_prime,phase)<gam_cutoff)
            eps=min([eps1,eps2]);   % take smaller eps
            cont=false;
        end
    else
        % neither limit of x2 results in eps in the frame
        eps=[];
        cont=false;
    end
end

% eps is empty if no chopper spurions in the frame
vi = 437.392.*sqrt(ei);
if ~isempty(t_bkgd)
    vf = x2_max./(min(t_bkgd)/10^6 - (x0+x1)./vi);
    if any(vf<=0), error('Incident energy too low - elastic line in background region'), end
else
    vf = x2_max./(tmax*(1-t0_frac)/10^6 - (x0+x1)./vi);
    if any(vf<=0), error('Incident energy too low - elastic line lies outside time frame'), end
end
eps=min([eps,min(ei-(vf/437.392).^2)]);   % lower of value set by spurions and set by background range or time frame.
if eps<0
    error('Calculated limits of energy integration range for vanadium are in energy gain!')
elseif eps<eps_resln*ei
    disp('WARNING: Calculated imits of energy integration range for vanadium lie very close to elastic line')
end
eps=[-0.5*(ei_min+ei_max)/2,eps];
