function [t,ei_prime,eps] = chopper_spurions (ei, hz, nrot, x0, x1, x2, eps_prime)
% Calculates the time-of-flight and apparent energy transfer for subsequent open position of the Fermi chopper.
%
% Input:
% -------
%   ei          Incident energy
%   hz          Frequency of Fermi chopper(Hz)
%   nrot        Multiples of rotation of chopper (e.g. 0.5 if 180 degree rotation)
%   x0          Moderator-chopper distance
%   x1          Chopper-sample distance
%   x2          Sample-detector distance
%   eps_prime   Energy transfer from later chopper opening
%
% Output:
% -------
%   t           Time-of-flight of the feature(s)
%   ei_prime    Incident energies of chopper spurions
%   eps         Energy transfer w.r.t. input ei chopper phasing
%
% Any one of the above can be an array, but only one.

numarr=~isscalar(ei) + ~isscalar(hz) + ~isscalar(nrot) + ~isscalar(x0) +...
       ~isscalar(x1) + ~isscalar(x2) + ~isscalar(eps_prime);
if numarr>1
    error('Only one input can be an array')
end 

vi = 437.392.*sqrt(ei);
v_open = x0*hz./nrot;
vi_prime = 1./(1./vi + 1./v_open);
vi_prime(vi_prime<0|~isfinite(vi_prime))=NaN;   % if nrot -ve, can have infinite or -ve vi_prime
ei_prime = (vi_prime/437.392).^2;
ef_prime = ei_prime - eps_prime;
ef_prime(ef_prime<=0)=0;                        % will result in infinite times
vf_prime = 437.392*sqrt(ef_prime);
vf = vf_prime.*v_open./(v_open+((x0+x1)./x2).*vf_prime);    % to ensure that vf_prime=0 does not cuase infinities
ef = (vf/437.392).^2;

t = 10^6*((x0+x1)./vi + x2./vf);
ei_prime = (vi_prime/437.392).^2;
eps = ei - ef;
