function rf=homer (dso,varargin)
% Perform analysis of raw file, with optional units conversion, normalisation by monitors or uA.hr,
% put on absolute intensity scale, normalise by detector solid angle etc.
%
% See on-line documentation for details

% Notes: should transfer checks on normalisation options to a point before the call to homer_pop

arglist = struct(... % argument names and default values
    'period',1,...
    'ei',[],...
    'fixei',[],...
    'background',[], ...
    'det_map','', ...
    'det_mask','', ...
    'mon_map','',...
    'mon_mask','',...
    'm_rebin',[],...
    'd_int',[],...
    'd_rebin',[],...
    'det_units','',...
    'mon_units','',...
    'normalisation',[],...
    'range',[],...
    'scale',[],...
    'nchunk',default_homer_nchunk,... % this seems to be the fastest of them all
    'solid',[],...
    'corr',0,...
    'abs',[],...
    'mass',[],...
    'RMM',[]);
flags = {'corr'};

% Pick up defaults for Homer from global defaults setting to update arglist
[dummy,arglist_def]=ixf_global_var('homer','get');
arglist = parse_arguments_arglist_defaults(arglist,flags,arglist_def);


% Parse arguments
[par,argout,present,filled] = parse_arguments(varargin,arglist,flags);



% Fill output runfile object
% =============================
rf=IXTrunfile;

% Add command that will reproduce the output
rf=create_population_command(rf,inputname(1),mfilename('fullpath'),mfilename,par,argout,filled);

% If there are run numbers or files defined in the par cell array, create entries in the dso
dso=create_dso_from_par(dso,par);
    

% If mask or map file(s) provided, add these to dso, over-riding any in the dso
% ------------------------------------------------------------------------------
detmap_created=false;
detmask_created=false;
detmap_tmpfile='temp_detmap.nxs';
detmask_tmpfile='temp_detmask.nxs';

monmap_created=false;
monmask_created=false;
monmap_tmpfile='temp_monmap.nxs';
monmask_tmpfile='temp_monmask.nxs';

if(filled.det_map)
    [dso,detmap_created]=map_prep(dso,detmap_tmpfile,'detmapfile',argout.det_map);
end
if(filled.det_mask)
    [dso,detmask_created]=mask_prep(dso,detmask_tmpfile,'detmaskfile',argout.det_mask);
end
if(filled.mon_map)
    [dso,monmap_created]=map_prep(dso,monmap_tmpfile,'monmapfile',argout.mon_map);
end
if(filled.mon_mask)
    [dso,monmask_created]=mask_prep(dso,monmask_tmpfile,'monmaskfile',argout.mon_mask);
end


% Parse incident energy and fill runfile object
% ----------------------------------------------
% This section may change the value and filled status of:
%   - ei
%   - d_rebin
%   - d_int
% 
if filled.d_int && filled.d_rebin
    error('ERROR: bad parameter input to homer, ''d_int'' and ''d_rebin'' are mutually exclusive')
end

if filled.ei && filled.fixei
    error('''ei'' and ''fixei'' are mutually exclusive keywords')
end

if filled.ei
    % Guess for ei if monochromatic, or instruction to force white beam anaysis
    if ischar(argout.ei) && strcmpi(argout.ei,'white')
        whitebeam=true;
        disp('white beam analysis')
        rf=homer_pop(rf,dso,argout,filled);
        
    elseif isnumeric(argout.ei) && isscalar(argout.ei) && argout.ei>0
        whitebeam=false;
        % Get monitors for incident energy determination
        tmp=read_ascii(IXTmask,'inst_maps:::getei_m_index.dat');  % file must contain two and only two monitor indicies
        ei_monitors=double(tmp.mask_array);
        if numel(ei_monitors)~=2
            error('Check contents of monitor index file for use in calculating ei')
        end
        % In case of energy transsfer, handle integration and rebinning with subtlety:
        % - if neither integration nor rebinning specified for energy transfer, use default energy binning
        % - if integration but corr=true, perform the integration later, as homer_pop does not perform corr
        if strcmpi(argout.det_units,'$w')
            [argout,filled,d_int_requested]=homer_energy_transfer_int_rebin_set(argout,filled);
        end
        rf=homer_pop(rf,dso,argout,filled,ei_monitors);
        
    else
        error('Input argument ''ei'' must be scalar numeric or the string ''white''')
    end
    
elseif(filled.fixei)
    % Force fixed monochromatic energy
    if isnumeric(argout.fixei) && isscalar(argout.fixei) && argout.fixei>0
        whitebeam=false;
        % Get monitors for incident energy determination
        tmp=read_ascii(IXTmask,'inst_maps:::getei_m_index.dat');  % file must contain two and only two monitor indicies
        ei_monitors=double(tmp.mask_array);
        if numel(ei_monitors)~=2
            error('Check contents of monitor index file for use in calculating ei')
        end
        % In case of energy transsfer, handle integration and rebinning with subtlety:
        % - if neither integration nor rebinning specified for energy transfer, use default energy binning
        % - if integration but corr=true, perform the integration later, as homer_pop does not perform corr
        if strcmpi(argout.det_units,'$w')
            [argout,filled,d_int_requested]=homer_energy_transfer_int_rebin_set(argout,filled);
        end
        rf=homer_pop(rf,dso,argout,filled,ei_monitors);
    else
        error('Input argument ''fixei'' must be scalar numeric or the string ''white''')
    end
    
else
    % White beam analysis only is possible
    whitebeam=true;
    argout.ei='white';
    filled.ei=true;
    disp('white beam analysis')
    rf=homer_pop(rf,dso,argout,filled);
    
end


% Perform normalisation if requested
% ----------------------------------
% if 'nonorm' then no normalisation will be performed
% This section may change the value and filled status of:
%   - scale

if filled.normalisation 
    if(isnumeric(argout.normalisation))
        % Normalisation by monitor data
        if isnumeric(argout.range)
            disp('Monitor integral normalisation')
            if ~filled.scale
                argout.scale=ixf_global_var('homer','get','mon_scale');
            end
            if isempty(argout.scale), argout.scale=1; end     % set scale to unity if no other source
            rf=mon_norm(rf,argout.normalisation,argout.range,argout.scale);
            
        elseif ischar(argout.range) && strcmpi(argout.range,'peak')
            if ~filled.scale
                argout.scale=ixf_global_var('homer','get','peak_scale');
            end
            if isempty(argout.scale), argout.scale=1; end     % set scale to unity if no other source
            if filled.ei && isnumeric(argout.ei)
                ei=argout.ei;   % Need to give the ei from which to get the peak areas
            elseif filled.fixei
                ei=argout.fixei;
            else
                error('Must have provided ''ei'' or ''fix_ei'' to normalise by monitor peak area')
            end
            disp('Monitor peak area normalisation')
            rf=peak_norm(rf,argout.normalisation,ei,argout.scale);
            
        else
            error('Must provide monitor integration range or define range as ''peak'' for monitor normalisation')
        end
        
    elseif ischar(argout.normalisation) && strcmpi(argout.normalisation,'uahr')
        % Normalisation by microamphours
        if present.range && filled.range     % Throw an error if explicitly gave a range in the keyword/value list
            error('Normalisation by microamphours incompatible with ''range'' keyword');
        end
        disp('Normalisation by microamphours')
        if ~filled.scale
            argout.scale=ixf_global_var('homer','get','uamp_scale');
        end
        if isempty(argout.scale), argout.scale=1; end     % set scale to unity if no other source
        rf=charge_norm(rf,argout.scale);
        
    else
        error('Check normalisation options are valid')
    end
end


% Other normalisation if requested
% ----------------------------------
% Solid angle normalisation
if filled.solid
    if isa(argout.solid,'IXTrunfile')
        rf=solidangle(rf,dso,argout.solid);
    else
        error('Solid angle normalisation information must be in an IXTrunfile object')
    end
end

% (kf/ki) and detector efficiency correction
if argout.corr
    if ~whitebeam && strcmpi(argout.det_units,'$w')
    	rf=corr(rf);
        if d_int_requested
            rf=integrate_x_det(rf);
        end
    else
        error('Detector efficiency correction only possible for monochromatic data on energy transfer scale')
    end
end

% Absolute normalisation if monochromatic run
if filled.abs
    if ~whitebeam
        if isnumeric(argout.abs)
            if filled.mass && filled.RMM && isnumeric(argout.mass) && isnumeric(argout.RMM)
                factor=argout.abs*(argout.mass/argout.RMM);
                rf=abs_norm(rf,factor);
            else
                error('Check sample mass and relative molecular mass are supplied and are both numeric')
            end
        else
            error('Absolute scaling factor must be numeric')
        end
    else
        error('Absolute normalisation only possible for monochromatic data')
    end
else
    if filled.RMM || filled.mass
        disp('No absolute normalisation requested: sample mass &/or relative molecular mass ignored.')
    end
end


% Delete temporary NeXus files
% ------------------------------
if(detmap_created)
    delete (detmap_tmpfile)
    dso=del_item(dso,'detmapfile');
end
if(detmask_created)
    delete (detmask_tmpfile)
    dso=del_item(dso,'detmaskfile');
end
if(monmap_created)
    delete (monmap_tmpfile)
    dso=del_item(dso,'monmapfile');
end
if(monmask_created)
    delete (monmask_tmpfile)
    dso=del_item(dso,'monmaskfile');
end
