function [rpath,ok,mess] = make_path_from_par(varargin)
% Construct a cell array of full raw file names from a cell array of 
%
%   >> rpath = make_path_from_par(par1,par2,...)
%
% Arguments can be:
%  - run numbers or array of run numbers  e.g.  [1234,11560:11565]  or  11567
%  - string representation of run numbers e.g. '[1234,11560:11565]' or '1234, 456' or '11567'
%  - character strings or character arrays of files
%       e.g. 'map01234.raw'    ['map01234.raw';'map01336.raw']
%  - cell array of strings
%       e.g.  {'map01234.raw','1345','inst_data:::map11456.raw','c:\temp\let00111.raw'}
%
%
% Advanced error handling:
% ------------------------
% Can avoid a fail, but report
%   >> [rpath,ok,mess] = make_path_from_par(par1,par2,...)
%

% Error handling
if nargout<3
    fail_on_error=true;
else
    fail_on_error=false;
end

% Get path, inst, ext (Note: this is an example where ixf_global_var is expected to return empty if a global doesn't exist)
def_path=ixf_global_var('data_source','get','path');
def_inst=ixf_global_var('data_source','get','inst');
def_ext=ixf_global_var('data_source','get','ext');

if isempty(def_ext)
    mess='Default extension type not defined';
    if fail_on_error;  error(mess);   else rpath={};    ok=false; 
       return, 
    end
end
if isempty(def_inst)
    mess='Default instrument name not defined';
    if fail_on_error; error(mess); else rpath={}; ok=false; return, end
end
if isempty(def_path)
    def_path='';
end

% Make full paths
[rpath,ok,mess]=make_raw_file_name(def_path, def_inst, def_ext, varargin{:});
if ~ok
    if fail_on_error, error(mess), else return, end
end

%-------------------------------------------------------------------------
function [rpath,ok,mess]=make_raw_file_name(def_path, def_inst, def_ext, varargin)
% Create file names for raw data from input

rpath={};
for i=1:numel(varargin)
    % Branch on argument type
    if isempty(varargin{i})
        rpath_add={};
    elseif isnumeric(varargin{i})   % numeric input (scalar or array)
        [rpath_add,ok,mess]=runno_to_raw_file_name(def_path,def_inst,def_ext,varargin{i});
        if ~ok, rpath={}; return, end
    elseif iscellstr(varargin{i})||ischar(varargin{i})
        [rpath_add,ok,mess]=strings_to_raw_file_name(def_path,def_inst,def_ext,varargin{i});
        if ~ok, rpath={}; return, end
    elseif iscell(varargin{i})
        [rpath_add,ok,mess]=make_raw_file_name(def_path,def_inst,def_ext,varargin{i}{:});
        if ~ok, return, end
    else
        rpath={}; ok=false; mess='Unrecognised input argument type';
        return
    end
    % Accumulate
    if ~isempty(rpath_add)
        rpath=[rpath,rpath_add];
    end
end
ok=true;
mess='';

%-------------------------------------------------------------------------
function [rpath,ok,mess]=strings_to_raw_file_name(def_path,def_inst,def_ext,arg)
% Construct file name from cell array of strings or character array

% Determine character input type
if iscellstr(arg)
    for i=1:numel(arg)
        if ~isempty(arg{i}) && numel(size(arg{i}))~=2   % character string or two dimensional character array
            rpath={}; ok=false; mess='Cellstr contents not a string or two dimensional character array';
            return
        end
    end
    rpath={};
    for i=1:numel(arg)
        if ~isempty(arg{i})
            [rpath_add,ok,mess]=char_to_raw_file_name(def_path,def_inst,def_ext,arg{i});
            if ~ok, rpath={}; return, end
            rpath=[rpath,rpath_add];
        end
    end
elseif ischar(arg) && numel(size(arg))==2
    [rpath,ok,mess]=char_to_raw_file_name(def_path,def_inst,def_ext,arg);
    if ~ok, rpath={}; return, end
elseif ischar(arg)
    rpath={}; ok=false; mess='Character array is not a string or two dimensional character array';
    return
else
    rpath={}; ok=false; mess='Unrecognised argument type - should never have reached here!!';
    return
end
ok=true;
mess='';

%-------------------------------------------------------------------------
function [rpath,ok,mess]=char_to_raw_file_name(def_path,def_inst,def_ext,arg)
% Construct cell array of file names from a 2D character array

rpath={};
for i=1:size(arg,1)
    runno=str2num(arg(i,:));
    if ~isempty(runno)
        [rpath_add,ok,mess]=runno_to_raw_file_name(def_path,def_inst,def_ext,runno);
        if ~ok, rpath={}; return, end
    else
        [rpath_add,ok,mess]=str_to_raw_file_name(def_path,def_inst,def_ext,arg(i,:));
        if ~ok, rpath={}; return, end
    end
    if ~isempty(rpath_add)
        rpath=[rpath,rpath_add];
    end
end
ok=true;
mess='';

%-------------------------------------------------------------------------
function [rpath,ok,mess]=str_to_raw_file_name(def_path,def_inst,def_ext,arg)
% Construct file name from string.
%  - if cannot find an extension, use the default extension
%  - if cannot find a path, use the default

rpath={};
if ~isempty(arg)
    arg=strtrim(arg);
    [path,name,ext]=fileparts(arg);
    if isempty(path)
        k=strfind(arg,':::');    % check is not a global path
        if isempty(k)
            arg=[def_path,arg];         % assume that def_path already has the correct file separator
        end
    end
    if isempty(ext)
        arg=[arg,'.',def_ext];
    end
    rpath={arg};
end
ok=true;
mess='';

%-------------------------------------------------------------------------
function [rpath,ok,mess]=runno_to_raw_file_name(def_path,def_inst,def_ext,runno)
% Construct file name from array of run numbers
n=numel(runno);
rpath=cell(1,n);
for i=1:n
    [num_str,ok,mess]=runchar(runno(i));
    if ~ok, rpath={}; return, end
    short_file = [def_inst,num_str,'.',def_ext];
    [rpath{i},is_generic]=full_extended_name(def_path,short_file);
    
    % check if proper case was selected 
    if is_generic
         file_2path = translate_read(rpath{i});
    else
        file_2path=rpath{i};
    end
     rpath{i}     = file_2path;
     if ~exist(file_2path,'file')&&(~is_generic)
        [short_file,lower_case]=other_case(short_file);
        full_file = full_extended_name(def_path,short_file);
         file_2path = translate_read(full_file);        
        if exist(file_2path,'file')
            rpath{i} = full_file;
            % change default case
              def_inst=ixf_global_var('data_source','get','inst');
              def_ext=ixf_global_var('data_source','get','ext');
              if lower_case
                  def_inst = lower(def_inst);
                  def_ext  = lower(def_ext);
              else
                  def_inst = upper(def_inst);
                  def_ext  = upper(def_ext);                 
              end           
              ixf_global_var('data_source','set','inst',def_inst);
              ixf_global_var('data_source','set','ext',def_ext);              
        end
    end
end
ok=true;
mess='';
function [fname,is_generic] = full_extended_name(path,name)
if regexp(path,':::$','once')
    fname = [path,name];
    is_generic=true;
else
    fname =fullfile(path,name);
    is_generic=false;    
end

function [different_case,low_case]=other_case(initial_case)

u_case=upper(initial_case);
l_case = lower(initial_case);

if strcmp(initial_case,u_case)
    different_case = l_case;
    low_case       = true;    
else
    different_case = u_case;   
    low_case       = false;
end
%-------------------------------------------------------------------------
function [num_str,ok,mess]=runchar(number)
% Convert run number into character string
% Assumes ISIS format i.e. 5 digits or 8 digits

if number<0
    num_str=0; ok=false; mess='Run number must be greater or equal to zero'; return
elseif number<=99999
    ndigits=5;
elseif number<=99999999
    ndigits=8;
else
    num_str=0; ok=false; mess='Run number too large'; return
end

num_str = num2str(number);
xlen = ndigits - length(num_str);
if xlen > 0
    num_str = [repmat('0', 1, xlen) num_str];
end
ok=true;
mess='';
