function indat=inst_defs(instname)
% returns an object with default parameters defined for the input
% instrument
% 
% >> indat = inst_defs(instname)
%
% instname may be any instrument name for which there is a text file
% located in 
%
% instrument_setup/<instname>.txt
%
% that defines the defaults for that instrument
%
% short names for the following instruments are also accepted if their long
% names exist as a text file in the instrument_setup folder
%
% Instrument Name       Text File Name       Short Name
%---------------------------------------------------------
%  MARI                     mari.txt            mar
%  MAPS                     maps.txt            map
%  MERLIN                   merlin.txt          mer
%   LET                       let.txt                let
instname=instrument_name(instname);
set(homgui_config,'instrument_name',instname);

config_file = get(homgui_config,'instr_config_file');
fid = fopen(config_file);    


if fid < 0   % Check file exists
    error('Instrument setup file does not exist')
end

while 1
    tline = fgetl(fid); % get line of text

    if ~ischar(tline), break, end  % break at end of file (tline = -1)

    tline = strtrim(tline); % remove trailing blanks
    datfield = ''; % make datfield and value blank by default.
    value = '';

    if ~isempty(tline) && ~strcmp(tline(1),'%')  % a % indicates comments, blank indicates line break.

        for i = 1:length(tline)   % search for the = sign,
            % use a loop instead of vectors because this is designed to be compiled
            if strcmp(tline(i),'=')

                % every thing to the left of the equals sign is the field,
                % to the right is the value
                datfield = tline(1:(i-1));
                value = tline((i+1):end);

                % remove any trailing blanks form the fields.
                value = strtrim(value);
                datfield = strtrim(datfield);

                %check through the datfield for further subfields
                datasubs = {};
                k = 0;
                for j = 1:length(datfield)
                    if strcmp(datfield(j-k),'.')
                        subfield = datfield(1:(j-k-1));      % take the left field
                        datfield = datfield((j-k+1):end);     % remove from datafield
                        datasubs = {datasubs{:}, subfield};  % add field to the cell array
                        k = j;
                    end
                end
                datasubs = {datasubs{:}, datfield};        % final field to be added


                % user may have put a , or ; at the end of the line
                if strcmp(value(end),';') || strcmp(value(end),',')
                    value(end) = [];  %remove the last character
                end

                value_num = str2num(value);                 % have a numerical equivilent to value ready.

                break           % no need to continue loop after = is found
            end
        end
        if isempty(value_num) && ~strcmp(value, '[]')   % the value is neither blank nor a number
            % remove any quotes from the string. 
            if strcmp(value(1), '''')
                value(1) = [];
            end
            if strcmp(value(end),'''')
                value(end)=[];
            end
        else
            value = value_num;                           % the value should be the numerical equivilent if possible
        end

        indat_temp = [];

        if ~ isempty(datasubs) && ~isempty(datasubs{1})
            indat_temp.(datasubs{end}) = value;      % put value into final field

            for j = 1:(length(datasubs)-1)      % fill the next parts
                index = length(datasubs) - j;
                indat_temp.(datasubs{index}) = indat_temp;   % get the next level
                indat_temp = rmfield(indat_temp, datasubs{index+1}); % delete the previous level
            end
            indat.(datasubs{1})=indat_temp.(datasubs{1});
        end
    end
end


fclose(fid);

% clear the empty fields
fn = fieldnames(indat);
for i=1:numel(fn)
    if numel(indat.(fn{i}))==0
        indat.(fn{i})=[];
    end
end

% Now run some checks that fields that should be there are there.
if isdeployed
    path = get(homer_config,'instruments_root_dir');
    file_name = fullfile(path,'required_fields.txt');
    if ~exist(file_name,'file')
        error('Homer:inst_defs','necessary configuration file : %s does not exsit',file_name);
    end
    fid = fopen(file_name);
else
    fid = fopen('instrument_setup/required_fields.txt'); % open the required fields file
end

if fid < 0   % Check file exists
    error('Instrument setup file does not exist')
end

while 1
    tline = fgetl(fid);                             % get line of text

    if ~ischar(tline), break, end                   % break at end of file (tline = -1)
    tline = strtrim(tline);                         % remove blanks

    if ~isempty(tline) && ~strcmp(tline(1),'%')        % a % indicates comments, blank indicates line break.

                if strcmp(tline(end),';') || strcmp(tline(end),',')
                    tline(end) = [];                % remove the last character
                end
                                                    % check the required field is a field in
                                                    % indat.
        if ~isempty(tline) && ~isfield(indat, tline)
            error(['Required field, ''' tline ''', not found in setup file'])
        end

    end
end

fclose(fid);
% Now the 'virgin' libisis distribution has the instrument definition files with
% paths either needing to be defined by the user (data paths) or auto generated by
% libisis (instrument parameters) This is to avoid the crazy paths being
% deployed to users.
% 
% deal with the special cases of having $User_define in the trunk version
% of the  definition file. 
if strcmp(indat.paramdir,'$Auto_gen') | strcmp(indat.datadir,'$User_define')
    uiwait(msgbox('This is probably the first time you have run Homer_ver0 with the instrument you selected, Critical Libisis paths not set need to be set please follow the onscreen instructions','FYI','modal'))
    uiwait(msgbox('You will now be prompted to set the raw file data directory path','FYI','modal'))
    indat.datadir = [uigetdir('Select a Raw file location'),filesep];
    
    uiwait(msgbox('You will now be prompted to set the directory path for saving .spe files','FYI','modal'));
    indat.spedir = [uigetdir('Select a Directory for saving .spe files'),filesep];
    
    uiwait(msgbox('You will now be prompted to set the data mapfile directory path or choose the LibIsis Default path autogeneration','FYI','modal'));
    choice_button=questdlg('Auto Generate path to map files?','Yes','No');
    switch choice_button,
        case 'Yes'
            disp('AutoGenerating map file directory path');
            instrument_files = get(homer_config,'instruments_root_dir');
            indat.mapdir=cat(2,instrument_files,filesep,instname,filesep);
        case 'No'
            indat.mapdir = uigetdir('Select a Map file Directory');
    end
    uiwait(msgbox('You will now be prompted to set the maskfile directory path or choose the LibIsis Default path autogeneration','FYI','modal'));
    choice_button=questdlg('Auto Generate path to mask files?','Yes','No');
    switch choice_button,
        case 'Yes'
            disp('AutoGenerating mask file directory path');
            instrument_files = get(homer_config,'instruments_root_dir');           
            indat.maskdir=cat(2,instrument_files,filesep,instname,filesep);
        case 'No'
            indat.maskdir = uigetdir('Select a Map file Directory');
    end
    
    if strcmp(indat.paramdir,'$Auto_gen');
        disp('AutoGenerating parameter file directory path');
        instrument_files = get(homer_config,'instruments_root_dir');                   
        indat.paramdir=cat(2,instrument_files,filesep,instname,filesep);
    end
    
    
%fid2 = fopen([files_location,filesep,instname,filesep,'homer_gui',filesep,instname,'.txt'],'w+');
config_file = get(homgui_config,'instr_config_file');
fid2 = fopen(config_file,'w+');
struct_names=fieldnames(indat);
for i = 1:length(struct_names)
    tmp=char(struct_names(i));
    to_process=getfield(indat,tmp);
   
   
    if isnumeric(to_process)
        if length(to_process)>1
            tmp2=to_process;
            out_str= [char(struct_names(i)), ' = ', '[',num2str(tmp2(1)),',',num2str(tmp2(2)),']'];
        else
            out_str= [char(struct_names(i)), ' = ',  num2str(to_process),';'];
        end
    end
    
    if islogical(to_process)
        tmplog=(to_process);
        if tmplog == 0
            tmplog_out='false';
        else
            tmplog_out='true';
        end
        out_str= [char(struct_names(i)), ' = ',  tmplog_out,';'];
    end
    
    if ischar(to_process)
        out_str= [char(struct_names(i)), ' = ', '''', to_process,'''',';'];
    end
     if isempty(to_process);
        out_str= [char(struct_names(i)), ' = ', '''','''',';'];
    end
   fprintf(fid2,'%s \n',out_str);
    %disp(out_str)
end
fclose(fid2)
end

% now write the defaults out to the instrument file



    


    
