function rf=spec(dso,varargin)
% Populate an IXTrunfile object with detector data only
%
%   >> runfile = spec(dso,run)
%   >> runfile = spec(dso,run,'keyword_1',arg1,'keyword_2',arg2,...)
%
% Optional keywords are:
%   'period'    Period number
%   'det_map'   map file, IXTmap object or , for a single workspace, an
%              array of spectrum numbers
%   'det_mask'  Mask file, IXTmask object, or an array of spectra to mask
%   'd_rebin'   Rebin parameters in the form:
%              [xlo,dx,xhi] or [x_1,dx_1,x_2,dx_2,...,x_n,dx_n,x_n+1]
%   'd_int'     Integration range [xlo,xhi]
%   'background' Background subtraction range [blo,bhi]
%
% NOTE: Cannot have both 'd_rebin' and 'd_int'


% NOTE: T.G.Perring, Feb 2010: For consistency with mon and pop_full,
% there should have been the line:
%       [present,argout] = getdefaults(present,argout);
% immediately following the parse_argument command. This is too dangerous,
% as it will transfer relevant homer defaults e.g. background or mask files,
% and so I have removed it from mon and pop_full in any case.

arglist = struct(... % argument names and default values
    'period',1,...
    'det_map','',...
    'det_mask','',...
    'd_rebin',[],...
    'd_int',[],...
    'background',[],...
    'nchunk',default_homer_nchunk); % this seems to be the fastest of them all


% Parse arguments
[par,argout,present,filled] = parse_arguments(varargin,arglist);

if (filled.d_int && filled.d_rebin)
    disp('Bad parameters input to spec: ''d_int'' and ''d_rebin'' are mutually exclusive');
    return
end

% Fill output runfile object
% --------------------------------
rf=IXTrunfile;

% Add command that will reproduce the output
rf=create_population_command(rf,inputname(1),mfilename('fullpath'),mfilename,par,argout,filled);

% If there are run numbers or files defined in the par cell array, create entries in the dso
dso=create_dso_from_par(dso,par);

% If mask or map file(s) provided, add these to dso, over-riding any in the dso
detmap_created=false;
detmask_created=false;

user_path=get(homer_config,'user_output');
detmap_tmpfile=fullfile(user_path,'temp_detmap.nxs');
detmask_tmpfile=fullfile(user_path,'temp_detmask.nxs');

if(filled.det_map);
    [dso,detmap_created]=map_prep(dso,detmap_tmpfile,'detmapfile',argout.det_map);
end
if(filled.det_mask)    
    [dso,detmask_created]=mask_prep(dso,detmask_tmpfile,'detmaskfile',argout.det_mask);
end

% Define the runfile as a diffraction instrument, because you may be
% able to do diffraction related units changes which don't require L2 or ei
dso=add_diffinst(dso);

% No normalisation performed, simply populates the det_data object
rf=det_pop(rf,dso,argout,filled);


% Delete temporary NeXus files
% ------------------------------
if(detmap_created)
    delete (detmap_tmpfile)
    dso=del_item(dso,'detmapfile');        
end
if(detmask_created)
    delete (detmask_tmpfile)
    dso=del_item(dso,'detmaskfile');        
end
