function map = IXTmap(varargin)
% Construct a libisis map object
%
%   >> map_object = IXTmap(speclist_cellarray)
%
%   >> map_object = IXTmap(workspace_numbers,speclist_cellarray)
%
% Input:
% -------
%   workspace_numbers   Numbers to be given to workspaces
%                       Default is [1:nw] where nw is the number of workspaces
%                      as defined by the number of lists in speclist_cellarray
%
%   speclist_cellarray  Cell array of numerical lists of the spectra in each
%                      workspace e.g.  {[1,5,11:15],[1025:2048]} defines two
%                      workspaces.
%
% Output:
% -------
%   map_object          Libisis map object (class is IXTmap)
%
%
% Examples:
%   >> my_mask = IXTmap([1:15,241:256])                % map with a single workspace
%   >> my_mask = IXTmap({[1,5,11:15],[1025:2048]})     % two workspaces
%
%   >> my_mask = IXTmap(999,[1:15,241:256])            % map with a single workspace, numbered 999
%   >> my_mask = IXTmap([11,21],{[1,5,11:15],[1025:2048]})     % two workspaces, numbered 11 and 21
%
%
% For experts: full constructor with IXTbase object (see online documnetation for more details):
%   >> map_object = IXTmap(base,work_no,total_spec,spec_ind,spec_no);


map.base = IXTbase;
map.work_no = int32(1);
map.total_spec = int32(1);
map.spec_ind = int32(1);
map.spec_no = int32(1);
map = class(map,'IXTmap');

% Case of empty constructor, just return
if nargin==0, return, end

% Just make copy if single input that is an IXTmask
if nargin==1 && isa(varargin{1},'IXTmap')
    map=varargin{1};
    return
end

% General input
if nargin ==2 || nargin==1
    if nargin==1 && (isnumeric(varargin{1}) || isa(varargin{1},'cell'))
        % Case of a single list of spectra or cell of spectrum number lists
        if isnumeric(varargin{1})
            cell=varargin(1);
        else
            cell=varargin{1};
        end
        n=numel(cell);
        work_no(1:n)=int32(1:n);
    elseif isnumeric(varargin{1}) && (isnumeric(varargin{2}) || isa(varargin{2},'cell'))
        % Case of workspace numbers and single list of spectra or cell of spectrum number lists
        if isnumeric(varargin{2})
            cell=varargin(2);
        else
            cell=varargin{2};
        end
        n=numel(cell);
        work_no=int32(varargin{1});
        nw=numel(unique(work_no));
        if nw~=numel(work_no)
            error('Workspace numbers are not all unique')
        elseif nw~=n
            error('Number of workspace numbers does not match number of spectrum number lists');
        end
    else
        error('Check input to IXTmap. Type >> help IXTmap for correct syntax');    
    end
    
    if ~isempty(cell)
        % Check spectrum number lists
        ns=zeros(1,n);
        for i=1:n
            if isnumeric(cell{i})
                ns(i)=numel(cell{i});
                cell{i}=int32(unique(cell{i}(:)')); % make row vectors of int32
                if numel(cell{i})~=ns(i) || ns(i)==0
                    error('The list of spectra in a workspace must contain at least one spectrum number and have no repeats')
                end
            else
                error('Not all spectrum number lists are numeric')
            end
        end
        % Fill arrays
        total_spec=int32(ns);
        spec_ind=int32([1,1+cumsum(ns(1:end-1))]);
        nstot=sum(ns);
        spec_no=zeros(1,nstot,'int32');
        for i=1:n
            spec_no(spec_ind(i):spec_ind(i)+total_spec(i)-1)=cell{i};
        end
    else
        error('Cell array containing spectrum number lists is empty. Type >> help IXTmap for correct syntax');
    end

    map = libisisexc('IXTmap','create',map,{IXTbase,work_no,total_spec,spec_ind,spec_no});

else
    if ~isa(varargin{1},'IXTbase'), varargin={IXTbase,varargin{:}}; end
    if numel(varargin)~=5, error('Check number of input arguments'), end
    map = libisisexc('IXTmap','create',map,varargin);

end
