function [dt,pk_fwhh]=pulse_width(fermi,ei,phase)
% Calculate st. dev. of chopper pulse width distribution (microseconds)
%
%   >> [dt,dt_fwhh]=pulse_width(fermi)        % pulse width for ei in Fermi chopper object
%   >> [dt,dt_fwhh]=pulse_width(fermi,ei)     % for an array of other ei
%   >> [dt,dt_fwhh]=pulse_width(fermi,ei,phase)
%
% Input:
% -------
%   fermi   IXTfermi_chopper object
%   ei      Incident energy (meV) (array or scalar)
%           If omitted or empty, use the ei value in the IXTfermi_chopper object
%   phase   if true, correctly phased; if false, 180 degrees out of phase
%
% Output:
% -------
%   dt      Standard deviation of pulse width (microseconds)
%   pk_fwhh FWHH at energy corresponding to maximum transmission

c_e_to_t=2286.271456507406;         % t(us)=c_e_to_t *distance(m)/sqrt(E(meV))
if nargin<2
    vi=1e6*sqrt(fermi.energy)/c_e_to_t; % incident velocity (m/s)
else
    vi=1e6*sqrt(ei)/c_e_to_t;           % incident velocity (m/s)
end

omega=2*pi*fermi.frequency;
s=2*omega*fermi.curvature;
pk_fwhh=fermi.slit_width/(2*fermi.radius*omega);
if phase
    gam=(2*fermi.radius/pk_fwhh)*abs(1/s-1./vi);
else
    gam=(2*fermi.radius/pk_fwhh)*abs(1/s+1./vi);
end

var=zeros(size(ei));
i1=gam<1;
var(i1)=(pk_fwhh^2/6)*((1-gam(i1).^4/10)./(1-gam(i1).^2/6));
i2=gam>=1&gam<4;
rtgam=sqrt(gam(i2));
var(i2)=(pk_fwhh^2/6)*(0.6*gam(i2).*((rtgam-2).^2).*(rtgam+8)./(rtgam+4));

dt=1e6*sqrt(var);
