function varargout=detector_data_to_nexus(file_in,file_out)
% Make minimal IXTdetector object from contents of detector.dat file.
% This file is one that fills the detector object internally to Homer
% with the same attributes as libisis presently (Jan 2010) takes from the
% raw file. It can therefore be used to create recalibrated detctor.dat
% for use with earlier raw files.
%
% To make Homer read in the nexus file created by this routine, insert
% into the part of the code that is creating the data source object (dso):
%
%   >> dso=add_item(dso,'inst_nxs:::this_file.nxs','detector','full_reference_detector')
%
% For afficionados: (TGP, Jan 2010)
%  - Homer assumes that the entry name of a reference detector file is
%  'full_reference_detector'.

% Get input detector.dat file name - prompt if file does not exist 
if ~exist('file_in','var')
    file_internal_in=getfile('*.dat');
    if isempty(file_internal_in), error('No file given'), end
else
    [file_internal_in,mess]=getfilecheck(file_in);
    if ~isempty(mess), error(mess), end
end

% Get output detector data file name - prompting if necessary
if ~exist('file_out','var')
    file_internal_out = putfile('*.nxs');
    if (isempty(file_internal_out))
        error ('No file given')
    end
elseif (nargin==2)
    file_internal_out = file_out;
end

% Read detector.dat file
win=read_detector_file(file_internal_in);

% Construct IXTdetector object
ndet=numel(win.det_no);

w.checksum=int32(1:ndet);
w.det_no=win.det_no;
w.geometry=make_geometry(ndet);
w.delay_time=win.delay_time;
w.dead_time=win.dead_time;
w.theta=win.theta;
w.L2=win.L2;
w.phi=win.phi;
w.group_index=int32(1:ndet);
w.det_type=int32(3)*ones(1,ndet,'int32');
w.det_he3=IXTdet_he3;
w.det_solid=IXTdet_solid;
w.type_index=zeros(1,ndet,'int32');
w.omega=ones(1,ndet);

% Get input to a set function (just like IXTgeometry, constructor doesn't sseem to work)
fnames={'checksum','det_no','geometry','delay_time','dead_time',...
        'theta','L2','phi','group_index','det_type','det_he3','det_solid','type_index','omega'};
w=[fnames;struct2cell(w)'];
w=w(:)';

% Create output object
det=set(IXTdetector,w{:});

% However, using the set method leaves w.base as false, so hand-craft this
det.base.valid=true;

% Save to file, making the entry name 
write_nxs(file_internal_out,det,'full_reference_detector');

% Return argument if requested
if nargout>=1
    varargout{1}=det;
end


%====================================================================================================
function wout=read_detector_file(file_in)
% Read full format ASCII detector.dat file into cell array

% Open file
fid=fopen(file_in);

% Read header info
tline=fgets(fid);
tline=fgets(fid);
temp = str2num(tline);
ndet=temp(1);
nuser=temp(2);
ncol=nuser+5;
tline=fgets(fid);

% Two formats acceptable:
% - 15 columns, last three are detector parameters
% - 19 columns, last four are detector parameters
if ncol==15
    idead=12;
elseif ncol==19
    idead=16;
else
    error('Unrecognised file format')
end

% Read array to the end, or until unable to read from file with specified format
fmt=repmat('%f',1,ncol);
tab = char(9);
w = textscan(fid, fmt, 'delimiter', [tab,',']);
fclose(fid)

% Pick out columns according to format
wout.det_no=int32(w{1}(:)');
wout.delay_time=w{2}(:)';
wout.dead_time=w{idead}(:)';
wout.theta=w{5}(:)';
wout.L2=w{3}(:)';
wout.phi=w{6}(:)';


%====================================================================================================
function geometry=make_geometry(ndet)
% Make dummy geometry object for a given number of detectors

% Note on using libisis functions: to create shape:
%   >> shape=IXTshape(IXTbase,int32(0),ones(1,ndet));
% is equivalent to:
%   >> shape = libisisexc('IXTshape','create',mm,{IXTbase,int32(0),ones(1,ndet)});
%
% Unfortunately, there appears to be an error in the Fortran for IXTgeometry that
% prevents the following working:
%
%   >> out=IXTgeometry(IXTbase,translation,orientation,elmt_to_shape,elmt_to_index,shape);
%
% This seems to be because the position of the status argument is incorrect in the Fortran. 
% Look at the subroutine definitions:
%   subroutine IXFcreate_geometry(geometry, translation, orientation,elmt_to_shape,elmt_to_index,shape,status)
%   subroutine IXFcreate_shape(shape, status, shape_type, dimensions)
%   subroutine IXFcreate_orientation(self, status, rotmat)
%   subroutine IXFcreate_translation(translation, status, vector)

translation=IXTtranslation(IXTbase,ones(3,ndet));
orientation=IXTorientation(IXTbase,ones(3,3,ndet));
elmt_to_shape=ones(1,ndet,'int32');
elmt_to_index=int32(1:ndet);
shape=IXTshape(IXTbase,int32(0),ones(1,ndet));

% Because the constructor doesnt seem to work as expected, use the set method instead
geometry=IXTgeometry;
geometry=set(geometry,'translation',translation,'orientation',orientation,...
    'elmt_to_shape',elmt_to_shape,'elmt_to_index',elmt_to_index,'shape',shape);

% However, using the set method leaves geometry.base as false, so hand-craft this
geometry.base.valid=true;
