function wout = spe (varargin)
% Create mslice/Tobyfit spe object
% 
%   >> w=spe(file)          % read from file
%   >> w=spe(structure)     % create from structure
%   >> w=spe(IXTdataset_1D,energies_array,[file]) % create from array of IXTdataset_1D;
%   >> w=spe(IXTdataset_2D,[file])        % create from IXTdataset_2D;

%
%  compartibility form: 
%   >> w=spe(structure,full_file_name) % create from a data structure with
%   the data fields only, where the file_name is given as a separate field
%   and the data has the structure below
%
% spe object has the following fields
%   data.filename   Name of file excluding path
%   data.filepath   Path to file including terminating file separator
%   data.S          [ne x ndet] array of signal values; masked pixels indicated by NaN
%   data.ERR        [ne x ndet] array of error values (st. dev.); masked pixels have value 0
%   data.en         Column vector of energy bin boundaries
%   data.Ei         incident energy from chopper or NaN for older data
%   format
%
%  nxspe files can also provide spe objects with additional field,
%  describing the contents of phx file; Such files should be processed
%  manually
%
%
%   if spe class is derived from array of IXTdataset_1D, it is assumed that
%   each dataset has the detectors coordinates along X-axis (Theta of a
%   detecor for example), and each IXTdataset_1D corresponds to its own energy. The
%   parameters then are:
%
%  IXTdataset_1D   arrays of the datasets with the signal vs detector data 
%                  x-axis is the detector's position (or tag for 2D detectors)
%  energies_array  the array of the energy bins correspondint to the each
%                  IXTdataset_1D. 
%  [file]          auxiliariy argument, which describes the filename to use
%                  by the routines writing spe file (if necessary)
%                  if this parameter is not supplied, an default filename
%                  is generated and writing (if any) occurs in working
%                  directory. 
%
%
%   IXTdataset_2D - this file has the structure similar to SPE data 
%                   we assume that the x-axis is the detector position and 
%                   y-axis is the energy. 
%
%   if the dataset contains histogram data, Y-axis data directly transferred
%   to SPE data. If dataset keeps linear data, the last edge of the energy
%   bin is guessed from the previous bin width. 
%
%   
% 
% Original author: T.G.Perring
%
% $Revision: 1781 $ ($Date: 2011-03-09 11:39:10 +0000 (Wed, 09 Mar 2011) $)
%
data = default_spe();
wout = class(data,classname); 
if nargin==0
    return
end

if isa(varargin{1},'IXTdataset_2d')
    specase='dataset2D';
    
    if nargin==2  % filename supplied
       [filepath,filename] = fileparts(varargin{2});         
    else
        filepath='';
        filename='';     
    end
elseif isa(varargin{1},'IXTdataset_1d')
    specase='dataset1D';
    if length(varargin{2}) ~= length(varargin{1})+1 || ~isa(varargin{2},'double')
        throw(MException('LIBISIS:spe:wrongArguments',' second parameter in case of the arrays of IXTdataset_1d has to be the array of energy bins with size of size(Dataset1D)+1'));       
    end
    if nargin==3  % filename supplied
        if ~ischar(varargin{3})
            throw(MException('LIBISIS:spe:wrongArguments',' third argiumennt has to be a file name'));                    
        end
       [filepath,filename] = fileparts(varargin{3});  
    else
        filepath='';
        filename='';     
    end
elseif isstruct(varargin{1}) %
    specase='speStruct'; 
elseif ischar(varargin{1})   % filename supplied
    specase='readFile';   
    if ~(ischar(varargin{1}) && length(size(varargin{1}))==2 && size(varargin{1},1)==1 )
            throw(MException('LIBISIS:spe:wrongArguments',' mailformed file name'));                            
    end
else
   throw(MException('LIBISIS:spe:wrongArguments',' call to spe class with unknown 1st argument'));    
end
    
switch specase
    case 'speStruct'
        field_names=fieldnames(varargin{1});
        if ~all(ismember({'S','ERR','en'},field_names))
            error('SPE:constructor','basic SPE srtucture does not have all necessary fields');
        end
        
        if(nargin==2)
          [wout.filepath,wout.filename] = fileparts(varargin{2});                   
        else
            if isfield(varargin{1},'filename')
              wout.filename=varargin{1}.filename;                            
            end
            if isfield(varargin{1},'filepath')            
              wout.filepath=varargin{1}.filepath; 
            end
            
        end
        wout.S  =varargin{1}.S;                        
        wout.ERR=varargin{1}.ERR;                                    
        wout.en =varargin{1}.en; 
        if isfield(varargin{1},'Ei')
            wout.Ei=varargin{1}.Ei;
        end
    case 'readFile'         
          [wout,ok,mess]=get_spe(wout,varargin{1});                
          if ~ok, error(mess); end;                
    case 'dataset2D'
        [filepath,filename] = check_IXTdataset_filename(filepath,filename,varargin{1}.title,'IXTdataset_2dSPE');
         % strip extension if any present
        [filepath,filename] = fileparts(fullfile(filepath,filename));

        wout.filename = filename;
        wout.filepath = filepath;
       %wout = IXTdataset_2d (IXTbase, title, spe.S, spe.ERR, s_axis, spe.en', x_axis, true, 1:size(spe.S,2), y_axis, false);
        wout.S  = varargin{1}.signal;
        wout.ERR= varargin{1}.error;

        % energy axis;
        xLen  =   length(varargin{1}.x);      
        if xLen+1==length(varargin{1}.signal)   % histogram data
            wout.en = varargin{1}.x;
        elseif xLen==length(varargin{1}.signal) % probably leftmost boundaries of the bins; the rightmost guessed
            wout.en = zeros(xLen+1,1);
            wout.en(1:xLen) = varargin{1}.x;
            wout.en(xLen+1) = wout.en(xLen)+(wout.en(xLen)-wout.en(xLen-1));
        end
   
    case 'dataset1D'
       [filepath,filename] = check_IXTdataset_filename(filepath,filename,varargin{1}.title,'IXTdataset_1dSPE');        
       wout.filename = filename;
       wout.filepath = filepath;
       [wout.S,wout.ERR]=extract_IXTDataset_1D_data(varargin{1});

       wout.en=varargin{2};
end
wout.use_hdf =get(homer_config,'use_hdf');                                
% strip extension if any present
[wout.filepath,wout.filename] = fileparts(fullfile(wout.filepath,wout.filename));


 
 % Make checkfields the ultimate arbiter of the validity of the class structure
[ok,mess,wout]=checkfields(wout);   
if ok, 
   return;
else
   throw(MException('LIBISIS:spe:wrongInputSrucure',' the srtucure of the SPE file is wrong %s',mess));       
end   

