function obj_out=read_nxs(object,filename,entry)
% Read a Libisis object from a NeXus file
%
%   >> output = read_nxs (IXTobject)                % prompts for file name
%   >> output = read_nxs (IXTobject, file_name)
%   >> output = read_nxs (IXTobject, file_name, entry_name)
%
%
%   IXTobject   Libisis object that defines the class that will be read
%               Most commonly, just use the constructor for that class
%               e.f. IXTmap, IXTfermi_chopper etc.
%
%   file_name   Name of NeXus file containing the object
%
%   entry_name  Name of a particular entry in a NeXus file, for example
%               if the NeXus file contains many entries of the requested
%               class type.
%
% Examples:
% ---------
%  Read in a runfile object that has been saved to a NeXus file:
%   >> rf = read_nxs(IXTrunfile, 'c:\my_data\ei50_psi90.nxs')
%
%  Read the entry 'sloppy' from a NeXus file holding the details of many Fermi choppers
%   >> chop_pars = read_nxs(IXTfermi_chopper,'inst_nxs:::fermi_chopper.nxs','sloppy')
%
%
% NOTE: for reasons of backwards compatibility, the first two arguments can be reversed
%       i.e. output = read_nxs (file_name, IXTobject) is valid

% Modified T.G.Perring 3 Feb 2010

if nargin==0
    error('Check arguments')
end

% Prompt for file
if nargin==1
    if isobject(object)
        filename=getfile;
        if isempty(filename), error('No file given'), end
    else
        error('Invalid object input')
    end
end

% Check input object and file name
if nargin>=2
    if isobject(filename) && ischar(object)
        tmp=object; object=filename; filename=tmp;  % reverse (backwards compatibility with old call syntax
    elseif ~(ischar(filename) && isobject(object))
        error('Invalid filename or object input')
    end
    if isempty(filename) || size(filename,1)~=1
        error('Filename must be a character string')
    end
end

% Check entry name
if nargin==3
    if ~(ischar(entry) && (isempty(entry) || size(entry,1)==1))  % accept empty character
        error('Entry name must be a character string')
    end
end

% Read file
disp(['Reading file: ',filename]);
fio=open(IXTfileio,translate_read(filename),1);
if nargin==3
    obj_out=read(object,fio,entry);
    fio=close(fio);
else
    obj_out=read(object,fio,'');
    fio=close(fio);
end
